import io
import json
import logging
import os
import platform
import sys
import zipfile

import requests

from qBitrr.config import FF_PROBE, FF_VERSION, FFPROBE_AUTO_UPDATE
from qBitrr.logger import run_logs


class FFprobeDownloader:
    def __init__(self):
        self.api = "https://ffbinaries.com/api/v1/version/latest"
        self.version_file = FF_VERSION
        self.logger = logging.getLogger("qBitrr.FFprobe")
        run_logs(self.logger)
        self.platform = platform.system()
        if self.platform == "Windows":
            self.probe_path = FF_PROBE.with_suffix(".exe")
        else:
            self.probe_path = FF_PROBE

    def get_upstream_version(self) -> dict:
        with requests.Session() as session:
            with session.get(self.api) as response:
                if response.status_code != 200:
                    self.logger.warning("Failed to retrieve ffprobe version from API.")
                    return {}
                return response.json()

    def get_current_version(self):
        try:
            with self.version_file.open(mode="r") as file:
                data = json.load(file)
            return data.get("version")
        except Exception:  # If file can't be found or read or parsed
            self.logger.warning("Failed to retrieve current ffprobe version.")
            return ""

    def update(self):
        if not FFPROBE_AUTO_UPDATE:
            return
        current_version = self.get_current_version()
        upstream_data = self.get_upstream_version()
        upstream_version = upstream_data.get("version")
        if upstream_version is None:
            self.logger.debug(
                "Failed to retrieve ffprobe version from API.'upstream_version' is None"
            )
            return
        probe_file_exists = self.probe_path.exists()
        if current_version == upstream_version and probe_file_exists:
            self.logger.debug("Current FFprobe is up to date.")
            return
        arch_key = self.get_arch()
        urls = upstream_data.get("bin", {}).get(arch_key)
        if urls is None:
            self.logger.debug("Failed to retrieve ffprobe version from API.'urls' is None")
            return
        ffprobe_url = urls.get("ffprobe")
        self.logger.debug("Downloading newer FFprobe: %s", ffprobe_url)
        self.download_and_extract(ffprobe_url)
        self.logger.debug("Updating local version of FFprobe: %s", upstream_version)
        self.version_file.write_text(json.dumps({"version": upstream_version}))
        try:
            os.chmod(self.probe_path, 0o777)
            self.logger.debug("Successfully changed permissions for ffprobe")
        except Exception as e:
            self.logger.debug("Failed to change permissions for ffprobe, %s", e)

    def download_and_extract(self, ffprobe_url):
        r = requests.get(ffprobe_url)
        z = zipfile.ZipFile(io.BytesIO(r.content))
        self.logger.debug("Extracting downloaded FFprobe to: %s", FF_PROBE.parent)
        z.extract(member=self.probe_path.name, path=FF_PROBE.parent)

    def get_arch(self):
        part1 = None
        is_64bits = sys.maxsize > 2**32
        part2 = "64" if is_64bits else "32"
        if self.platform == "Windows":
            part1 = "windows-"
        elif self.platform == "Linux":
            part1 = "linux-"
            machine = platform.machine()
            if machine == "armv6l":
                part2 = "armhf"
            elif ("arm" in machine and is_64bits) or machine == "aarch64":
                part2 = "arm64"
            # Else just 32/64, Not armel - because just no
        elif self.platform == "Darwin":
            part1 = "osx-"
            part2 = "64"
        if part1 is None:
            raise RuntimeError(
                "You are running in an unsupported platform, "
                "if you expect this to be supported please open an issue on GitHub "
                "https://github.com/Feramance/qBitrr."
            )

        return part1 + part2



from peewee import BooleanField, CharField, DateTimeField, IntegerField, Model, TextField


class FilesQueued(Model):
    EntryId = IntegerField(primary_key=True, null=False, unique=True)


class MoviesFilesModel(Model):
    Title = CharField()
    Monitored = BooleanField()
    TmdbId = IntegerField()
    Year = IntegerField()
    EntryId = IntegerField(unique=True)
    Searched = BooleanField(default=False)
    MovieFileId = IntegerField()
    IsRequest = BooleanField(default=False)
    QualityMet = BooleanField(default=False)
    Upgrade = BooleanField(default=False)
    CustomFormatScore = IntegerField(null=True)
    MinCustomFormatScore = IntegerField(null=True)
    CustomFormatMet = BooleanField(default=False)
    Reason = TextField(null=True)


class EpisodeFilesModel(Model):
    EntryId = IntegerField(primary_key=True)
    SeriesTitle = TextField(null=True)
    Title = TextField(null=True)
    SeriesId = IntegerField(null=False)
    EpisodeFileId = IntegerField(null=True)
    EpisodeNumber = IntegerField(null=False)
    SeasonNumber = IntegerField(null=False)
    AbsoluteEpisodeNumber = IntegerField(null=True)
    SceneAbsoluteEpisodeNumber = IntegerField(null=True)
    AirDateUtc = DateTimeField(formats=["%Y-%m-%d %H:%M:%S.%f"], null=True)
    Monitored = BooleanField(null=True)
    Searched = BooleanField(default=False)
    IsRequest = BooleanField(default=False)
    QualityMet = BooleanField(default=False)
    Upgrade = BooleanField(default=False)
    CustomFormatScore = IntegerField(null=True)
    MinCustomFormatScore = IntegerField(null=True)
    CustomFormatMet = BooleanField(default=False)
    Reason = TextField(null=True)


class SeriesFilesModel(Model):
    EntryId = IntegerField(primary_key=True)
    Title = TextField(null=True)
    Monitored = BooleanField(null=True)
    Searched = BooleanField(default=False)
    Upgrade = BooleanField(default=False)
    MinCustomFormatScore = IntegerField(null=True)


class MovieQueueModel(Model):
    EntryId = IntegerField(unique=True)
    Completed = BooleanField(default=False)


class EpisodeQueueModel(Model):
    EntryId = IntegerField(unique=True)
    Completed = BooleanField(default=False)


class TorrentLibrary(Model):
    Hash = TextField(null=False)
    Category = TextField(null=False)
    AllowedSeeding = BooleanField(default=False)
    Imported = BooleanField(default=False)
    AllowedStalled = BooleanField(default=False)
    FreeSpacePaused = BooleanField(default=False)



from __future__ import annotations

import logging
import time
from logging import Logger

import coloredlogs

from qBitrr.config import (
    AUTO_PAUSE_RESUME,
    COMPLETED_DOWNLOAD_FOLDER,
    CONFIG,
    CONSOLE_LOGGING_LEVEL_STRING,
    COPIED_TO_NEW_DIR,
    ENABLE_LOGS,
    FAILED_CATEGORY,
    FREE_SPACE,
    HOME_PATH,
    IGNORE_TORRENTS_YOUNGER_THAN,
    LOOP_SLEEP_TIMER,
    NO_INTERNET_SLEEP_TIMER,
    PING_URLS,
    RECHECK_CATEGORY,
    SEARCH_LOOP_DELAY,
    TAGLESS,
)

__all__ = ("run_logs",)

TRACE = 5
VERBOSE = 7
NOTICE = 23
HNOTICE = 24
SUCCESS = 25


class VerboseLogger(Logger):
    def _init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        if self.name.startswith("qBitrr"):
            self.set_config_level()

    def success(self, message, *args, **kwargs):
        if self.isEnabledFor(SUCCESS):
            self._log(SUCCESS, message, args, **kwargs)

    def hnotice(self, message, *args, **kwargs):
        if self.isEnabledFor(HNOTICE):
            self._log(HNOTICE, message, args, **kwargs)

    def notice(self, message, *args, **kwargs):
        if self.isEnabledFor(NOTICE):
            self._log(NOTICE, message, args, **kwargs)

    def verbose(self, message, *args, **kwargs):
        if self.isEnabledFor(VERBOSE):
            self._log(VERBOSE, message, args, **kwargs)

    def trace(self, message, *args, **kwargs):
        if self.isEnabledFor(TRACE):
            self._log(TRACE, message, args, **kwargs)

    def set_config_level(self):
        self.setLevel(CONSOLE_LOGGING_LEVEL_STRING)


logging.addLevelName(SUCCESS, "SUCCESS")
logging.addLevelName(HNOTICE, "HNOTICE")
logging.addLevelName(NOTICE, "NOTICE")
logging.addLevelName(VERBOSE, "VERBOSE")
logging.addLevelName(TRACE, "TRACE")
logging.setLoggerClass(VerboseLogger)


def getLogger(name: str | None = None):
    return VerboseLogger.manager.getLogger(name) if name else logging.root


logging.getLogger = getLogger


logger = logging.getLogger("qBitrr.Misc")


HAS_RUN = False


def run_logs(logger: Logger) -> None:
    global HAS_RUN
    try:
        configkeys = {f"qBitrr.{i}" for i in CONFIG.sections()}
        key_length = max(len(max(configkeys, key=len)), 10)
    except BaseException:
        key_length = 10
    coloredlogs.install(
        logger=logger,
        level=logging._nameToLevel.get(CONSOLE_LOGGING_LEVEL_STRING),
        fmt="[%(asctime)-15s] [pid:%(process)8d][tid:%(thread)8d] "
        f"%(levelname)-8s: %(name)-{key_length}s: %(message)s",
        level_styles={
            "trace": {"color": "black", "bold": True},
            "debug": {"color": "magenta", "bold": True},
            "verbose": {"color": "blue", "bold": True},
            "info": {"color": "white"},
            "notice": {"color": "cyan"},
            "hnotice": {"color": "cyan", "bold": True},
            "warning": {"color": "yellow", "bold": True},
            "success": {"color": "green", "bold": True},
            "error": {"color": "red"},
            "critical": {"color": "red", "bold": True},
        },
        field_styles={
            "asctime": {"color": "green"},
            "process": {"color": "magenta"},
            "levelname": {"color": "red", "bold": True},
            "name": {"color": "blue", "bold": True},
            "thread": {"color": "cyan"},
        },
        reconfigure=True,
    )
    if HAS_RUN is False:
        HAS_RUN = True
        log_debugs(logger)


def log_debugs(logger):
    logger.debug("Log Level: %s", CONSOLE_LOGGING_LEVEL_STRING)
    logger.debug("Ping URLs:  %s", PING_URLS)
    logger.debug("Script Config:  Logging=%s", ENABLE_LOGS)
    logger.debug("Script Config:  FailedCategory=%s", FAILED_CATEGORY)
    logger.debug("Script Config:  RecheckCategory=%s", RECHECK_CATEGORY)
    logger.debug("Script Config:  Tagless=%s", TAGLESS)
    logger.debug("Script Config:  CompletedDownloadFolder=%s", COMPLETED_DOWNLOAD_FOLDER)
    logger.debug("Script Config:  FreeSpace=%s", FREE_SPACE)
    logger.debug("Script Config:  LoopSleepTimer=%s", LOOP_SLEEP_TIMER)
    logger.debug("Script Config:  SearchLoopDelay=%s", SEARCH_LOOP_DELAY)
    logger.debug("Script Config:  AutoPauseResume=%s", AUTO_PAUSE_RESUME)
    logger.debug("Script Config:  NoInternetSleepTimer=%s", NO_INTERNET_SLEEP_TIMER)
    logger.debug("Script Config:  IgnoreTorrentsYoungerThan=%s", IGNORE_TORRENTS_YOUNGER_THAN)


if COPIED_TO_NEW_DIR is False and not HOME_PATH.joinpath("config.toml").exists():
    logger.warning(
        "Config.toml should exist in '%s', in a future update this will be a requirement.",
        HOME_PATH,
    )
    time.sleep(5)
if COPIED_TO_NEW_DIR:
    logger.warning("Config.toml new location is %s", HOME_PATH)
    time.sleep(5)
run_logs(logger)
